package com.qmth.distributed.print.api.obe;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.core.rateLimit.annotation.RateLimit;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.obe.been.report.ObeCourseRequirementDto;
import com.qmth.teachcloud.obe.been.result.ObeCourseRequirementListResult;
import com.qmth.teachcloud.obe.been.result.ScoreResult;
import com.qmth.teachcloud.obe.entity.ObeCourseRequirementReport;
import com.qmth.teachcloud.obe.service.ObeCourseRequirementReportService;
import com.qmth.teachcloud.obe.service.TRExamStudentService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * obe专业达成度报表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "obe专业达成度报表Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_OBE)
public class ObeReportController {
    private final static Logger log = LoggerFactory.getLogger(ObeReportController.class);

    @Resource
    ObeCourseRequirementReportService obeCourseRequirementReportService;

    @Resource
    TRExamStudentService trExamStudentService;

    @Resource
    RedisUtil redisUtil;

    @ApiOperation(value = "毕业要求达成度重新计算")
    @RequestMapping(value = "/requirements/again/calculate", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = ObeCourseRequirementListResult.class)})
    @Aac(rateLimit = @RateLimit(count = 1, period = 1000L))
    public Result requirementsAgainCalculate(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId) {
        String lockKey = SystemConstant.REDIS_OBE_RADAR_AGAIN_DATA_PREFIX + cultureProgramId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_OBE_RADAR_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在重新计算数据，请稍候再试！");
        }
        ObeCourseRequirementDto obeCourseRequirementResultDto = null;
        try {
            obeCourseRequirementResultDto = obeCourseRequirementReportService.calculate(cultureProgramId);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(obeCourseRequirementResultDto);
    }

    @ApiOperation(value = "毕业要求达成度报表列表")
    @RequestMapping(value = "/requirements/list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = ObeCourseRequirementListResult.class)})
    public Result requirementsList(@ApiParam(value = "专业ID") @RequestParam(required = false) Long professionalId,
                                   @ApiParam(value = "培养方案名称") @RequestParam(required = false) String cultureProgramName,
                                   @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                   @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(obeCourseRequirementReportService.obeCourseRequirementReportList(new Page<>(pageNumber, pageSize), professionalId, cultureProgramName));
    }

    @ApiOperation(value = "毕业要求达成度雷达报表")
    @RequestMapping(value = "/requirements/radar/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度雷达报表", response = ScoreResult.class)})
    public Result requirementsRadarReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId) {
        String lockKey = SystemConstant.REDIS_OBE_RADAR_DATA_PREFIX + cultureProgramId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_OBE_RADAR_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在计算数据，请稍候再试！");
        }
        ObeCourseRequirementDto obeCourseRequirementResultDto = null;
        try {
            ObeCourseRequirementReport obeCourseRequirementReport = obeCourseRequirementReportService.findByCultureProgramId(cultureProgramId);
            if (Objects.nonNull(obeCourseRequirementReport) && Objects.nonNull(obeCourseRequirementReport.getRequirementRadar())) {
                obeCourseRequirementResultDto = JSONObject.parseObject(obeCourseRequirementReport.getRequirementRadar(), ObeCourseRequirementDto.class);
            } else {
                obeCourseRequirementResultDto = obeCourseRequirementReportService.calculate(cultureProgramId);
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(obeCourseRequirementResultDto);
    }

    @ApiOperation(value = "毕业要求达成度详情报表")
    @RequestMapping(value = "/requirements/detail/report", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度详情报表", response = ScoreResult.class)})
    public Result requirementsDetailReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId) {
        String lockKey = SystemConstant.REDIS_OBE_RADAR_DETAIL_DATA_PREFIX + cultureProgramId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_OBE_RADAR_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在计算详情数据，请稍候再试！");
        }
        List<ObeCourseRequirementDto> obeCourseRequirementDtoList = null;
        try {
            ObeCourseRequirementReport obeCourseRequirementReport = obeCourseRequirementReportService.findByCultureProgramId(cultureProgramId);
            if (Objects.nonNull(obeCourseRequirementReport) && Objects.nonNull(obeCourseRequirementReport.getRequirementDetail())) {
                obeCourseRequirementDtoList = JSONArray.parseArray(obeCourseRequirementReport.getRequirementDetail(), ObeCourseRequirementDto.class);
            } else {
                obeCourseRequirementDtoList = obeCourseRequirementReportService.calculateDetail(cultureProgramId);
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(obeCourseRequirementDtoList);
    }

    @ApiOperation(value = "毕业要求达成度学生重新计算")
    @RequestMapping(value = "/student_requirement/again/calculate", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    @Aac(rateLimit = @RateLimit(count = 1, period = 1000L))
    public Result requirementsStudentAgainCalculate(@ApiParam(value = "培养方案ID") @RequestParam(required = false) Long cultureProgramId) {
        String lockKey = SystemConstant.REDIS_OBE_RADAR_STUDENT_DATA_PREFIX + cultureProgramId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_OBE_RADAR_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在重新计算数据，请稍候再试！");
        }
        ObeCourseRequirementDto obeCourseRequirementResultDto = null;
        try {
            obeCourseRequirementResultDto = obeCourseRequirementReportService.calculate(cultureProgramId);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(obeCourseRequirementResultDto);
    }

    @ApiOperation(value = "毕业要求达成度学生报表-列表")
    @RequestMapping(value = "/student_requirement/page", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    public Result requirementsStudentPage(@ApiParam(value = "培养方案ID") @RequestParam(required = false) Long cultureProgramId,
                                          @ApiParam(value = "学号/姓名") @RequestParam(required = false) String studentCodeOrName,
                                          @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                          @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(trExamStudentService.getRequirementsStudentPage(cultureProgramId, studentCodeOrName, pageNumber, pageSize));
    }

    @ApiOperation(value = "毕业要求达成度学生报表-查看详情")
    @RequestMapping(value = "/student_requirement/detail", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "毕业要求达成度学生报表", response = ScoreResult.class)})
    public Result requirementsStudentReport(@ApiParam(value = "培养方案ID", required = true) @RequestParam Long cultureProgramId,
                                            @ApiParam(value = "学号", required = true) @RequestParam String studentCode,
                                            @ApiParam(value = "学期") @RequestParam(required = false) Long semesterId) {
        return ResultUtil.ok(trExamStudentService.getRequirementsStudentReport(cultureProgramId, studentCode, semesterId));
    }
}
