package com.qmth.teachcloud.mark.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.QuestionType;
import com.qmth.teachcloud.mark.bean.vo.parseCard.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 题卡格式中试卷结构解析
 */
public class CardParseUtils {
    private final static Logger log = LoggerFactory.getLogger(CardParseUtils.class);

    /**
     * 解析题卡结构
     *
     * @param content 题卡内容
     */
    public static List<Struct> parseCardContent(String content) {
        JSONObject jsonObject = JSON.parseObject(content, JSONObject.class);
        List<Pages> pages = JSON.parseArray(jsonObject.getString("pages"), Pages.class);
        List<StructData> structDataList = new ArrayList<>();
        String curTopicId = "0";
        int i = 1;
        for (Pages page : pages) {
            for (Columns column : page.getColumns()) {
                for (Elements topic : column.getElements()) {
                    if (topic.getParent() == null || curTopicId.equals(topic.getParent().getId())) {
                        continue;
                    }
                    curTopicId = topic.getParent().getId();
                    int questionsCount = 1, startNumber = 1;
                    if (topic.getType().equals("COMPOSITION")) {
                        long composition = structDataList.stream().filter(m -> m.getCardTopicType().equals("COMPOSITION") && m.getMainNumber() == topic.getParent().getTopicNo() * 1).count();
                        if (composition > 0) {
                            continue;
                        }
                    } else {
                        questionsCount = topic.getParent().getQuestionsCount();
                        startNumber = topic.getParent().getStartNumber() == null ? 1 : topic.getParent().getStartNumber();
                    }

                    QuestionType questionType = getQuestionType(topic);
                    if (questionType == null) {
                        continue;
                    }
                    StructData structData = new StructData();
                    structData.setType(questionType.getValue());
                    structData.setMainTitle(questionType.getName());
                    structData.setObjective(questionType.getObjective());
                    structData.setqType(questionType.getObjective() ? "objective" : "subjective");
                    structData.setMainNumber(topic.getParent().getTopicNo() * 1);
                    structData.setMainTitle(topic.getParent().getTopicName());
                    structData.setQuestionsCount(questionsCount);
                    structData.setStartNumber(startNumber);
                    structData.setCardTopicType(topic.getType());
                    if ("FILL_QUESTION".equals(topic.getType())) {
                        structData.setOptionCount(topic.getOptionCount());
                    }
                    // 偶数页
                    if (i % 2 == 0) {
                        structData.setPaperIndex(i / 2);
                        structData.setPageIndex(i % 2 + 2);
                    }
                    // 奇数页
                    else {
                        structData.setPaperIndex(i / 2 + 1);
                        structData.setPageIndex(i % 2);
                    }
                    structDataList.add(structData);
                }
            }
            i++;
        }

        Map<String, String> answerMap = new HashMap<>();
        if (jsonObject.containsKey("answers")) {
            answerMap = JSON.parseObject(jsonObject.getString("answers"), Map.class);
        }
        List<Struct> structList = new ArrayList<>();
        for (StructData structData : structDataList) {
            int startNumber = structData.getStartNumber();
            for (int i1 = 0; i1 < structData.getQuestionsCount(); i1++) {
                Struct struct = new Struct();
                struct.setqType(structData.getqType());
                struct.setMainNumber(structData.getMainNumber());
                struct.setSubNumber(startNumber + i1);
                if (!answerMap.isEmpty()) {
                    String key = struct.getMainNumber() + "-" + struct.getSubNumber();
                    if (answerMap.containsKey(key)) {
                        Answer answer = JSON.parseObject(JSON.toJSONString(answerMap.get(key)), Answer.class);
                        struct.setAnswer(StringUtils.isBlank(answer.getAnswer()) ? null : answer.getAnswer().replaceAll(",", ""));
                        struct.setScore(answer.getScore());
                    }
                }
                struct.setMainTitle(structData.getMainTitle());
                struct.setType(structData.getType());
                struct.setObjective(structData.getObjective());
                struct.setTypeName(structData.getTypeName());
                struct.setOptionCount(structData.getOptionCount());
                struct.setPaperIndex(structData.getPaperIndex());
                struct.setPageIndex(structData.getPageIndex());
                structList.add(struct);
            }
        }

        structList.stream().sorted(Comparator.comparingInt(Struct::getMainNumber).thenComparing(Struct::getSubNumber));
        return structList;
    }

    /**
     * 获取题卡页数
     *
     * @param content 题卡内容
     * @param isPaper 是否返回张数（页数/2）
     */
    public static int calcPaperCount(String content, Boolean isPaper) {
        if (StringUtils.isNotBlank(content)) {
            JSONObject jsonObject = JSON.parseObject(content, JSONObject.class);
            List<Pages> pages = JSON.parseArray(jsonObject.getString("pages"), Pages.class);
            if (CollectionUtils.isNotEmpty(pages)) {
                int pageSize = pages.size();
                return isPaper ? pageSize / 2 : pageSize;
            }
        }
        return 0;

    }

    private static QuestionType getQuestionType(Elements topic) {
        String type = topic.getType();
        if ("COMPOSITION".equals(type) || "EXPLAIN".equals(type)) {
            return QuestionType.findByValue(5);
        }
        if ("FILL_LINE".equals(type)) {
            return QuestionType.findByValue(4);
        }
        if ("FILL_QUESTION".equals(type)) {
            if (topic.getBoolean()) {
                return QuestionType.findByValue(3);
            }
            if (topic.getMultiply()) {
                return QuestionType.findByValue(2);
            }
            return QuestionType.findByValue(1);
        }
        return null;
    }


    public static String parseAreaInfo(String content) {
        JSONObject jsonObject = JSON.parseObject(content, JSONObject.class);
        List<Pages> pages = JSON.parseArray(jsonObject.getString("pages"), Pages.class);
        List<Map<String, Object>> areaList = new ArrayList<>();
        int i = 1;
        for (Pages page : pages) {
            if (page.getExchange() != null && CollectionUtils.isNotEmpty(page.getExchange().getInfoArea())) {
                for (List<Double> infoArea : page.getExchange().getInfoArea()) {
                    if (infoArea != null) {
                        Map<String, Object> map = new HashMap<>();
                        map.put("i", i);
                        map.put("x", infoArea.get(0));
                        map.put("y", infoArea.get(1));
                        map.put("w", infoArea.get(2));
                        map.put("h", infoArea.get(3));
                        areaList.add(map);
                    }
                }
            }
            i++;
        }
        return CollectionUtils.isNotEmpty(areaList) ? JSON.toJSONString(areaList) : null;
    }

    public static void main(String[] args) {
        String content = "{\"version\":\"2.0.0\",\"cardType\":\"STANDARD\",\"cardConfig\":{\"id\":\"476860503277178880\",\"createId\":\"476858560215519232\",\"createTime\":1704633203628,\"updateId\":\"476858560215519232\",\"updateTime\":1713231215510,\"schoolId\":\"1\",\"orgId\":\"476857569122455552\",\"name\":\"题卡规则1\",\"examNumberStyle\":\"PRINT\",\"examNumberDigit\":10,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"discipline\":true,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"enable\":true,\"name\":\"学号\",\"disabled\":true,\"ordinal\":1,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"siteNumber\",\"enable\":false,\"name\":\"座位号\",\"disabled\":false,\"ordinal\":2},{\"code\":\"courseName\",\"enable\":true,\"name\":\"课程名称\",\"disabled\":true,\"ordinal\":3,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"courseCode\",\"enable\":true,\"name\":\"课程代码\",\"disabled\":true,\"ordinal\":4,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"paperNumber\",\"enable\":true,\"name\":\"试卷编号\",\"disabled\":true,\"ordinal\":5,\"contentHtml\":\"<i>试</i> <i>卷</i> <i>编</i> <i>号</i>\"},{\"code\":\"examDate\",\"enable\":true,\"name\":\"考试日期\",\"disabled\":true,\"ordinal\":6,\"contentHtml\":\"<i>考</i> <i>试</i> <i>日</i> <i>期</i>\"},{\"code\":\"examTime\",\"enable\":false,\"name\":\"考试时间\",\"disabled\":true,\"ordinal\":7},{\"code\":\"examPlace\",\"enable\":true,\"name\":\"校区（考点）\",\"disabled\":true,\"ordinal\":8,\"contentHtml\":\"<i>校</i> <i>区</i> <i>（</i> <i>考</i> <i>点</i> <i>）</i>\"},{\"code\":\"examRoom\",\"enable\":true,\"name\":\"考试教室（考场）\",\"disabled\":true,\"ordinal\":9,\"contentHtml\":\"<i>考</i> <i>试</i> <i>教</i> <i>室</i> <i>（</i> <i>考</i> <i>场</i> <i>）</i>\"},{\"code\":\"collegeName\",\"enable\":false,\"name\":\"学院\",\"disabled\":true,\"ordinal\":10},{\"code\":\"majorName\",\"enable\":true,\"name\":\"专业\",\"disabled\":false,\"ordinal\":11,\"contentHtml\":\"<i>专</i> <i>业</i>\"},{\"code\":\"teachClazzName\",\"enable\":true,\"name\":\"教学班\",\"disabled\":true,\"ordinal\":12,\"contentHtml\":\"<i>教</i> <i>学</i> <i>班</i>\"},{\"code\":\"clazzName\",\"enable\":false,\"name\":\"行政班\",\"disabled\":false,\"ordinal\":13},{\"code\":\"teacherName\",\"enable\":true,\"name\":\"任课老师\",\"disabled\":true,\"ordinal\":14,\"contentHtml\":\"<i>任</i> <i>课</i> <i>老</i> <i>师</i>\"}],\"fillFields\":\"\",\"titleRule\":\"${schoolName}\",\"firstLevelSubheading\":\"一级\",\"secondLevelSubheading\":\"二级\",\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"enable\":true,\"remark\":\"\",\"fillNumber\":10,\"orgIds\":null,\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"showForbidArea\":false,\"makeMethod\":\"SELF\",\"aOrB\":false,\"cardTitle\":\"测试学校_1\"},\"pages\":[{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-0-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-0-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-0-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[{\"type\":\"CARD_HEAD\",\"x\":0,\"y\":0,\"w\":703,\"h\":594,\"cardTitle\":\"测试学校_1\",\"firstLevelSubheading\":\"一级\",\"secondLevelSubheading\":\"二级\",\"aOrB\":false,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"fillNumber\":10,\"examNumberStyle\":\"PRINT\",\"businessParams\":[],\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"columnNumber\":2,\"isSimple\":false,\"sign\":\"head\",\"id\":\"element-vfbpiq9of1r180t8\",\"createId\":\"476858560215519232\",\"createTime\":1704633203628,\"updateId\":\"476858560215519232\",\"updateTime\":1713231215510,\"schoolId\":\"1\",\"orgId\":\"476857569122455552\",\"name\":\"题卡规则1\",\"examNumberDigit\":10,\"discipline\":true,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"enable\":true,\"name\":\"学号\",\"disabled\":true,\"ordinal\":1,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"siteNumber\",\"enable\":false,\"name\":\"座位号\",\"disabled\":false,\"ordinal\":2},{\"code\":\"courseName\",\"enable\":true,\"name\":\"课程名称\",\"disabled\":true,\"ordinal\":3,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"courseCode\",\"enable\":true,\"name\":\"课程代码\",\"disabled\":true,\"ordinal\":4,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"paperNumber\",\"enable\":true,\"name\":\"试卷编号\",\"disabled\":true,\"ordinal\":5,\"contentHtml\":\"<i>试</i> <i>卷</i> <i>编</i> <i>号</i>\"},{\"code\":\"examDate\",\"enable\":true,\"name\":\"考试日期\",\"disabled\":true,\"ordinal\":6,\"contentHtml\":\"<i>考</i> <i>试</i> <i>日</i> <i>期</i>\"},{\"code\":\"examTime\",\"enable\":false,\"name\":\"考试时间\",\"disabled\":true,\"ordinal\":7},{\"code\":\"examPlace\",\"enable\":true,\"name\":\"校区（考点）\",\"disabled\":true,\"ordinal\":8,\"contentHtml\":\"<i>校</i> <i>区</i> <i>（</i> <i>考</i> <i>点</i> <i>）</i>\"},{\"code\":\"examRoom\",\"enable\":true,\"name\":\"考试教室（考场）\",\"disabled\":true,\"ordinal\":9,\"contentHtml\":\"<i>考</i> <i>试</i> <i>教</i> <i>室</i> <i>（</i> <i>考</i> <i>场</i> <i>）</i>\"},{\"code\":\"collegeName\",\"enable\":false,\"name\":\"学院\",\"disabled\":true,\"ordinal\":10},{\"code\":\"majorName\",\"enable\":true,\"name\":\"专业\",\"disabled\":false,\"ordinal\":11,\"contentHtml\":\"<i>专</i> <i>业</i>\"},{\"code\":\"teachClazzName\",\"enable\":true,\"name\":\"教学班\",\"disabled\":true,\"ordinal\":12,\"contentHtml\":\"<i>教</i> <i>学</i> <i>班</i>\"},{\"code\":\"clazzName\",\"enable\":false,\"name\":\"行政班\",\"disabled\":false,\"ordinal\":13},{\"code\":\"teacherName\",\"enable\":true,\"name\":\"任课老师\",\"disabled\":true,\"ordinal\":14,\"contentHtml\":\"<i>任</i> <i>课</i> <i>老</i> <i>师</i>\"}],\"fillFields\":\"\",\"titleRule\":\"${schoolName}\",\"enable\":true,\"remark\":\"\",\"orgIds\":null,\"pageSize\":\"A3\",\"columnGap\":20,\"showForbidArea\":false,\"makeMethod\":\"SELF\",\"key\":\"4ihoqe3g2pb15n0g\",\"isCovered\":false},{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":10,\"content\":\"\",\"typeName\":\"客观题\",\"isColumnFirst\":false,\"sign\":\"objective\",\"id\":\"element-jltuakhg8tiiuvvj\",\"key\":\"bop1c28orsj41dog\"},{\"id\":\"element-7g6ka16oekqntv7l\",\"topicNo\":1,\"topicName\":\"1\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"uiq4o4bj0cup8j0o\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":5,\"questionGap\":8,\"groupGap\":27,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-goeo3np8i6afsnc8\",\"topicNo\":1,\"topicName\":\"1\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"3r0s1f2ma9tgsquo\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":0},{\"id\":\"element-f12nvj18447u9pvv\",\"topicNo\":2,\"topicName\":\"2\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":2,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":true,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"dkpt3v3hmp7d8g7o\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":7,\"questionGap\":8,\"groupGap\":33,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-bhri4mlmm3d65h28\",\"topicNo\":2,\"topicName\":\"2\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":2,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":true,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"lqnj6hdg5q45oagg\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":1}]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.0504413619,0.0267379679,0.0151324086,0.0142602496],[0.8738965952,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.0504413619,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[{\"field\":\"examNumber\",\"area\":[0.0504413619,0.1310160428,0.2184741488,0.3048128342]}],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[[0.0504413619,0.0534759358,0.4432534678,0.5294117647]],\"fill_area\":[{\"field\":\"absent\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.3394546028,0.5608845811,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"breach\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.4483764187,0.5608845811,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":1,\"options\":[[0.0687263556,0.6310160428,0.0113493064,0.008912656],[0.0838587642,0.6310160428,0.0113493064,0.008912656],[0.0989911728,0.6310160428,0.0113493064,0.008912656],[0.1141235813,0.6310160428,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":2,\"options\":[[0.0687263556,0.6470588235,0.0113493064,0.008912656],[0.0838587642,0.6470588235,0.0113493064,0.008912656],[0.0989911728,0.6470588235,0.0113493064,0.008912656],[0.1141235813,0.6470588235,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":3,\"options\":[[0.0687263556,0.6631016043,0.0113493064,0.008912656],[0.0838587642,0.6631016043,0.0113493064,0.008912656],[0.0989911728,0.6631016043,0.0113493064,0.008912656],[0.1141235813,0.6631016043,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":4,\"options\":[[0.0687263556,0.679144385,0.0113493064,0.008912656],[0.0838587642,0.679144385,0.0113493064,0.008912656],[0.0989911728,0.679144385,0.0113493064,0.008912656],[0.1141235813,0.679144385,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":5,\"options\":[[0.0687263556,0.6951871658,0.0113493064,0.008912656],[0.0838587642,0.6951871658,0.0113493064,0.008912656],[0.0989911728,0.6951871658,0.0113493064,0.008912656],[0.1141235813,0.6951871658,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":2,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":6,\"options\":[[0.157629256,0.6310160428,0.0113493064,0.008912656],[0.1727616646,0.6310160428,0.0113493064,0.008912656],[0.1878940731,0.6310160428,0.0113493064,0.008912656],[0.2030264817,0.6310160428,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":7,\"options\":[[0.157629256,0.6470588235,0.0113493064,0.008912656],[0.1727616646,0.6470588235,0.0113493064,0.008912656],[0.1878940731,0.6470588235,0.0113493064,0.008912656],[0.2030264817,0.6470588235,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":8,\"options\":[[0.157629256,0.6631016043,0.0113493064,0.008912656],[0.1727616646,0.6631016043,0.0113493064,0.008912656],[0.1878940731,0.6631016043,0.0113493064,0.008912656],[0.2030264817,0.6631016043,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":9,\"options\":[[0.157629256,0.679144385,0.0113493064,0.008912656],[0.1727616646,0.679144385,0.0113493064,0.008912656],[0.1878940731,0.679144385,0.0113493064,0.008912656],[0.2030264817,0.679144385,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":10,\"options\":[[0.157629256,0.6951871658,0.0113493064,0.008912656],[0.1727616646,0.6951871658,0.0113493064,0.008912656],[0.1878940731,0.6951871658,0.0113493064,0.008912656],[0.2030264817,0.6951871658,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":3,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":1,\"options\":[[0.0687263556,0.752228164,0.0113493064,0.008912656],[0.0838587642,0.752228164,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":2,\"options\":[[0.0687263556,0.7682709447,0.0113493064,0.008912656],[0.0838587642,0.7682709447,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":3,\"options\":[[0.0687263556,0.7843137255,0.0113493064,0.008912656],[0.0838587642,0.7843137255,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":4,\"options\":[[0.0687263556,0.8003565062,0.0113493064,0.008912656],[0.0838587642,0.8003565062,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":5,\"options\":[[0.0687263556,0.816399287,0.0113493064,0.008912656],[0.0838587642,0.816399287,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":4,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":6,\"options\":[[0.131147541,0.752228164,0.0113493064,0.008912656],[0.1462799496,0.752228164,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":7,\"options\":[[0.131147541,0.7682709447,0.0113493064,0.008912656],[0.1462799496,0.7682709447,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":8,\"options\":[[0.131147541,0.7843137255,0.0113493064,0.008912656],[0.1462799496,0.7843137255,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":9,\"options\":[[0.131147541,0.8003565062,0.0113493064,0.008912656],[0.1462799496,0.8003565062,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":10,\"options\":[[0.131147541,0.816399287,0.0113493064,0.008912656],[0.1462799496,0.816399287,0.0113493064,0.008912656]],\"recog_info\":[]}]}],\"answer_area\":[],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}},{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-1-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-1-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-1-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.1109709962,0.0267379679,0.0151324086,0.0142602496],[0.9344262295,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.9344262295,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[],\"fill_area\":[],\"answer_area\":[],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}}],\"answers\":{}}";
        List<Struct> structList = CardParseUtils.parseCardContent(content);
        System.out.println(JSON.toJSONString(structList));

        String info = CardParseUtils.parseAreaInfo(content);
        System.out.println(info);
    }

}
