package com.qmth.teachcloud.obe.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.obe.been.result.report.audit.*;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import com.qmth.teachcloud.obe.entity.ObeCourseOutlineAuditReport;
import com.qmth.teachcloud.obe.entity.ObeCourseTarget;
import com.qmth.teachcloud.obe.enums.CourseEvaluationTypeEnum;
import com.qmth.teachcloud.obe.mapper.ObeCourseOutlineAuditReportMapper;
import com.qmth.teachcloud.obe.service.ObeCourseOutlineAuditReportService;
import com.qmth.teachcloud.obe.service.ObeCourseOutlineService;
import com.qmth.teachcloud.obe.service.ObeCourseTargetService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程目标达成情况评价依据合理性审核表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-10-08
 */
@Service
public class ObeCourseOutlineAuditReportServiceImpl extends ServiceImpl<ObeCourseOutlineAuditReportMapper, ObeCourseOutlineAuditReport> implements ObeCourseOutlineAuditReportService {

    @Resource
    private ObeCourseOutlineService obeCourseOutlineService;

    @Resource
    private ObeCourseTargetService obeCourseTargetService;

    @Override
    public AuditReport getAuditReport(Long courseOutlineId, boolean rationality) {
        AuditReport result = new AuditReport();
        ObeCourseOutline obeCourseOutline = obeCourseOutlineService.getById(courseOutlineId);
        if (Objects.isNull(obeCourseOutline)) {
            throw ExceptionResultEnum.ERROR.exception("课程大纲不存在");
        }

        ObeCourseOutlineAuditReport dbReport = this.getOne(new QueryWrapper<ObeCourseOutlineAuditReport>().lambda().eq(ObeCourseOutlineAuditReport::getCourseOutlineId, courseOutlineId).last(SystemConstant.LIMIT1));
        if (Objects.nonNull(dbReport)) {
            // 曾生成过报告 -> 组装报告
            Long auditSign = dbReport.getWeightSettingSign();
            Long outlineSign = obeCourseOutline.getWeightSettingSign();

            String courseBasicInfoStr = dbReport.getCourseBasicInfo();
            String courseTargetInfoStr = dbReport.getCourseTargetInfo();
            String rationalityEvaluationStr = dbReport.getRationalityEvaluation();

            CourseBasicInfo courseBasicInfo = JSON.parseObject(courseBasicInfoStr, CourseBasicInfo.class);
            CourseTargetInfo courseTargetInfo = JSON.parseObject(courseTargetInfoStr, CourseTargetInfo.class);
            RationalityEvaluation rationalityEvaluation = null;
            if (SystemConstant.strNotNull(rationalityEvaluationStr)) {
                rationalityEvaluation = JSON.parseObject(rationalityEvaluationStr, RationalityEvaluation.class);
            }

            if (!Objects.equals(auditSign, outlineSign)) {
                // 权重设置有变化 -> 重新生成权重信息
                courseTargetInfo = this.getCourseTargetInfo(courseOutlineId);
            }
            if (rationality) {
                if (Objects.isNull(rationalityEvaluation)) {
                    // 如果缺少合理性信息 -> 重新生成
                    rationalityEvaluation = this.getRationalityEvaluation(courseOutlineId);
                }
            } else {
                rationalityEvaluation = null;
            }
            if (Objects.isNull(courseTargetInfo.getDate())) {
                courseTargetInfo.setDate(System.currentTimeMillis());
            }
            if (Objects.nonNull(rationalityEvaluation)) {
                if (Objects.isNull(rationalityEvaluation.getAuditDate())) {
                    rationalityEvaluation.setAuditDate(System.currentTimeMillis());
                }
                if (Objects.isNull(rationalityEvaluation.getEvaluateDate())) {
                    rationalityEvaluation.setEvaluateDate(System.currentTimeMillis());
                }
            }

            result.setCourseBasicInfo(courseBasicInfo);
            result.setCourseTargetInfo(courseTargetInfo);
            result.setRationalityEvaluation(rationalityEvaluation);
        } else {
            // 还未生成过报告 -> 直接生成
            result.setCourseBasicInfo(this.getCourseBasicInfo(courseOutlineId));
            result.setCourseTargetInfo(this.getCourseTargetInfo(courseOutlineId));
            RationalityEvaluation rationalityEvaluation;
            if (rationality) {
                rationalityEvaluation = this.getRationalityEvaluation(courseOutlineId);
                result.setRationalityEvaluation(rationalityEvaluation);
            }
        }
        result.setCourseOutlineId(courseOutlineId);
        return result;
    }

    @Transactional
    @Override
    public CourseBasicInfo getCourseBasicInfo(Long courseOutlineId) {
        return this.baseMapper.findCourseBasicInfo(courseOutlineId);
    }

    @Override
    public CourseTargetInfo getCourseTargetInfo(Long courseOutlineId) {
        CourseTargetInfo result = new CourseTargetInfo();
        BigDecimal procedureTotalWeight = new BigDecimal(0);
        BigDecimal finalityTotalWeight = new BigDecimal(0);
        List<CourseEvaluate> courseEvaluateList = new ArrayList<>();

        List<ObeCourseTarget> obeCourseTargetList = obeCourseTargetService.list(
                new QueryWrapper<ObeCourseTarget>().lambda().eq(ObeCourseTarget::getObeCourseOutlineId, courseOutlineId));
        List<ObeCourseWeightInfo> obeCourseWeightDatasource = this.baseMapper.findCourseWeightInfo(courseOutlineId);

        for (ObeCourseTarget obeCourseTarget : obeCourseTargetList) {
            Long targetId = obeCourseTarget.getId();
            String targetName = obeCourseTarget.getTargetName();
            // 过程性评价数据
            List<ObeCourseWeightInfo> obeCourseWeighList = obeCourseWeightDatasource.stream()
                    .filter(e -> targetId.equals(e.getTargetId())).collect(Collectors.toList());
            List<EvaluateInfo> procedureEvaluateList = new ArrayList<>();
            List<EvaluateInfo> finalityEvaluateList = new ArrayList<>();
            for (ObeCourseWeightInfo obeCourseWeightInfo : obeCourseWeighList) {
                BigDecimal weight = obeCourseWeightInfo.getWeight();
                String evaluation = obeCourseWeightInfo.getEvaluation();
                CourseEvaluationTypeEnum type = obeCourseWeightInfo.getType();
                EvaluateInfo evaluateInfo = new EvaluateInfo();
                evaluateInfo.setContent(evaluation);
                evaluateInfo.setScore(weight);
                if (CourseEvaluationTypeEnum.CUSTOM.equals(type)) {
                    procedureEvaluateList.add(evaluateInfo);
                    procedureTotalWeight = procedureTotalWeight.add(weight);
                } else {
                    finalityEvaluateList.add(evaluateInfo);
                    finalityTotalWeight = finalityTotalWeight.add(weight);
                }
            }

            CourseEvaluate courseEvaluate = new CourseEvaluate();
            courseEvaluate.setName(targetName);
            courseEvaluate.setProcedureEvaluateList(procedureEvaluateList);
            courseEvaluate.setFinalityEvaluateList(finalityEvaluateList);
            courseEvaluateList.add(courseEvaluate);
        }
        result.setCourseEvaluateList(courseEvaluateList);
        result.setProceduralTotalScore(procedureTotalWeight);
        result.setFinalityTotalScore(finalityTotalWeight);
        result.setDate(System.currentTimeMillis());
        return result;
    }

    @Override
    public RationalityEvaluation getRationalityEvaluation(Long courseOutlineId) {
        RationalityEvaluation result = new RationalityEvaluation();
        result.initRationalityEvaluationDetailList();
        result.setEvaluateDate(System.currentTimeMillis());
        result.setAuditDate(System.currentTimeMillis());
        return result;
    }

    @Transactional
    @Override
    public Long saveReport(AuditReport auditReport) {
        Long courseOutlineId = auditReport.getCourseOutlineId();
        ObeCourseOutline obeCourseOutline = obeCourseOutlineService.getById(courseOutlineId);
        if (Objects.isNull(obeCourseOutline)) {
            throw ExceptionResultEnum.ERROR.exception("未找到课程大纲");
        }

        Long weightSettingSign = obeCourseOutline.getWeightSettingSign();
        CourseBasicInfo courseBasicInfo = auditReport.getCourseBasicInfo();
        CourseTargetInfo courseTargetInfo = auditReport.getCourseTargetInfo();
        RationalityEvaluation rationalityEvaluation = auditReport.getRationalityEvaluation();

        if (Objects.isNull(courseTargetInfo.getCourseManager())) {
            courseTargetInfo.setDate(null);
        }
        if (Objects.nonNull(rationalityEvaluation)) {
            if (Objects.isNull(rationalityEvaluation.getAuditor())) {
                rationalityEvaluation.setAuditDate(null);
            }
            if (Objects.isNull(rationalityEvaluation.getProfessionalManager())) {
                rationalityEvaluation.setEvaluateDate(null);
            }
        }

        Long id = null;
        ObeCourseOutlineAuditReport dbReport = this.getOne(new QueryWrapper<ObeCourseOutlineAuditReport>().lambda().eq(ObeCourseOutlineAuditReport::getCourseOutlineId, courseOutlineId).last(SystemConstant.LIMIT1));
        if (Objects.nonNull(dbReport)){
            id = dbReport.getId();
        }
        ObeCourseOutlineAuditReport obeCourseOutlineAuditReport = new ObeCourseOutlineAuditReport();
        if (Objects.isNull(id)) {
            // 新增
            id = SystemConstant.getDbUuid();
            obeCourseOutlineAuditReport.setId(id);
            obeCourseOutlineAuditReport.setCourseOutlineId(courseOutlineId);
            obeCourseOutlineAuditReport.setWeightSettingSign(weightSettingSign);
            obeCourseOutlineAuditReport.setCourseBasicInfo(JSON.toJSONString(courseBasicInfo));
            obeCourseOutlineAuditReport.setCourseTargetInfo(JSON.toJSONString(courseTargetInfo));
            if (Objects.nonNull(rationalityEvaluation)) {
                obeCourseOutlineAuditReport.setRationalityEvaluation(JSON.toJSONString(rationalityEvaluation));
            }
            this.save(obeCourseOutlineAuditReport);
        } else {
            // 编辑
            UpdateWrapper<ObeCourseOutlineAuditReport> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(ObeCourseOutlineAuditReport::getId, id)
                    .set(ObeCourseOutlineAuditReport::getCourseOutlineId, courseOutlineId)
                    .set(ObeCourseOutlineAuditReport::getWeightSettingSign, weightSettingSign)
                    .set(ObeCourseOutlineAuditReport::getCourseBasicInfo, JSON.toJSONString(courseBasicInfo))
                    .set(ObeCourseOutlineAuditReport::getCourseTargetInfo, JSON.toJSONString(courseTargetInfo));
            if (Objects.nonNull(rationalityEvaluation)) {
                updateWrapper.lambda().set(ObeCourseOutlineAuditReport::getRationalityEvaluation, JSON.toJSONString(rationalityEvaluation));
            }
            this.update(updateWrapper);
        }
        return id;
    }
}
